﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace InvokeMethodByName
{
	public static class MethodFinder
	{
		/// <summary>
		/// search the method into the class
		/// </summary>
		/// <param name="module_class">the class (System.Type)</param>
		/// <returns>true if successful</returns>
		public static MethodInfo FindClassMethod( Type module_class, string method_name)
		{
			if( module_class == null || string.IsNullOrEmpty( method_name))
				return null;

			MethodInfo[]	methods;

			try
			{
				methods = module_class.GetMethods();
			}
			catch (Exception)
			{
				return null;
			}

			foreach (MethodInfo method in methods)
			{
				if( string.Compare( method.Name, method_name, true) == 0)
					return method;
			}

			return null;
		}

		/// <summary>
		/// find the method into the module
		/// </summary>
		/// <param name="module">module info (reflection)</param>
		/// <returns>true if successful</returns>
		public static MethodInfo FindModuleMethod(Module module, string method_name)
		{
			if( module == null || string.IsNullOrEmpty( method_name))
				return null;

			Type[]		module_types;
			MethodInfo	method;

			try
			{
				module_types = module.GetTypes();
			}

			/// ReflectionTypeLoadException: is thrown by the Module.GetTypes method 
			/// if any of the classes in a module cannot be loaded.
			catch (ReflectionTypeLoadException refex)
			{
				module_types	= refex.Types;
			}
			catch (Exception)
			{
				return null;
			}

			foreach (Type module_class in module_types)
			{
				if (module_class != null)
				{
					method = FindClassMethod( module_class, method_name);
					if( method != null)
						return method;
				}
			}

			return null;
		}

		/// <summary>
		/// find the method into the assembly
		/// </summary>
		/// <param name="asm">the assembly (reflection)</param>
		/// <returns>true if successful</returns>
		public static MethodInfo FindAssemblyMethod(Assembly asm, string method_name)
		{
			if( asm == null || string.IsNullOrEmpty( method_name))
				return null;

			MethodInfo	method;

			foreach (Module module in asm.GetModules())
			{
				method	= FindModuleMethod( module, method_name);
				if( method != null)
					return method;
			}
			return null;
		}


		/// <summary>
		/// search the method into all loaded assemblies
		/// according to current settings (include GAC...)
		/// </summary>
		/// <returns>true if successful</returns>
		public static MethodInfo FindMethod( string method_name)
		{
			if( string.IsNullOrEmpty( method_name))
				return null;

			MethodInfo	method;

			foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies())
			{
				/// skip assembly if it was not loaded into execution context
				if( asm.ReflectionOnly == true)
					continue;
				/// skip Global Cache assemblies
				if (asm.GlobalAssemblyCache == true)
					continue;

				method	= FindAssemblyMethod( asm, method_name);
				if( method != null)
					return method;
			}
			return null;
		}

	}

}